<?php

/**
 * Account setting form for itsyou.online
 *
 * Page callback for 'admin/config/services/itsyouonline/account'
 *
 * @return array
 *   Form generated by system_settings_form().
 */
function itsyouonline_admin_settings_account($form_state) {
  global $base_url;

  $form = array();

  $form['itsyouonline_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable itsyou.online integration'),
    '#default_value' => variable_get('itsyouonline_enabled', 0),
  );

  // Account related settings

  $form['account'] = array(
    '#type' => 'fieldset',
    '#title' => t('Account settings'),
  );

  $form['account']['itsyouonline_client_id'] = array(
    '#type' => 'textfield',
    '#title' => 'Client ID',
    '#default_value' => variable_get('itsyouonline_client_id', ''),
    '#description' => t('The client identifier (organization identifier) issued by itsyou.online.'),
    '#required' => TRUE,
  );
 
  $client_secret = variable_get('itsyouonline_client_secret', '');

  if (empty($client_secret) || (isset($_GET['edit_secret']) && $_GET['edit_secret'] == 1)) {
    $form['account']['itsyouonline_client_secret'] = array(
      '#type' => 'textfield',
      '#title' => 'Client secret',
      '#default_value' => $client_secret,
      '#description' => t('The client secret issued by itsyou.online.'),
      '#required' => TRUE,
    );
  }
  else {
    $form['account']['itsyouonline_show_secret'] = array(
      '#type' => 'item',
      '#title' => 'Client secret',
      '#markup' => t('Click <a href="@link">here</a> to edit the client secret', array('@link' => url('admin/config/services/itsyouonline', array('query' => array('edit_secret' => 1))))),
    );
  }

  // User registration related settings

  $form['registration'] = array(
    '#type' => 'fieldset',
    '#title' => t('User account registration'),
  );

  $form['registration']['itsyouonline_username_pattern'] = array(
    '#type' => 'textfield',
    '#default_value' => variable_get('itsyouonline_username_pattern', '{itsyou.username}'),
    '#title' => t('Automatically create an account for users connecting via itsyou.online.'),
    '#description' => t('Username pattern to create drupal user account. Use following pattern {itsyou.username}, {itsyou.email}, {itsyou.firstname}, {itsyou.lastname}'),
  );

  $form['registration']['itsyouonline_auto_create_account'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('itsyouonline_auto_create_account', 0),
    '#title' => t('Automatically create an account for users connecting via itsyou.online.'),
    '#description' => t('By enabling this option, a new user account will be automatically (without asking drupal registration) created for users who connect via itsyou.online if the itsyou.online account is not yet linked to a Drupal account.'),
  );
  
  $base_url_parts = explode('/', $base_url);
  $email_domain = $base_url_parts[2];

  $form['registration']['itsyouonline_auto_create_account_email'] = array(
    '#type' => 'radios',
    '#default_value' => variable_get('itsyouonline_auto_create_account_email', 'random'),
    '#title' => t('If accounts are created automatically, and the email address is not provided by itsyou.online or is is already exists in drupal, what value should be used as email?'),
    '#description' => t('Under some conditions itsyou.online may not provide the email address of a user. This is the case when the user has not yet confirmed  their email address to itsyou.online. However, Drupal requires an email address when creating an account. If you do not need the email address of the user, then it is safe to select that a random email address should be generated. If you need to send emails to the user, then it is recommended to ask the user to enter his email address.'),
    '#options' => array(
      'random' => t('Generate a random email address (this will be [itsyou.online username] + [unique string if required]@!domain).', array('!domain' => $email_domain)),
      'form' => t('Show a form and ask the user to type his email address.'),
    ),
    '#states' => array(
      // Hide the settings when the itsyouonline_auto_create_account checkbox is disabled.
      'invisible' => array(
       ':input[name="itsyouonline_auto_create_account"]' => array('checked' => FALSE),
      ),
    ),
    '#required' => TRUE,
  );

  $form['registration']['itsyouonline_auto_create_account_username'] = array(
    '#type' => 'radios',
    '#default_value' => variable_get('itsyouonline_auto_create_account_username', 'random'),
    '#title' => t('If accounts are created automatically, and username is empty or it is already exists in drupal, what value should be used as username?'),
    '#description' => t('If username (after processing the username pattern of above field), already exist in drupal database what should be done since username should be unique in drupal.'),
    '#options' => array(
      'random' => t('Generate a random username (this will be either a email or like [username field pattern] + [unique string if required]).', array('!domain' => $email_domain)),
      'form' => t('Show a form and ask the user to type/change the username.'),
    ),
    '#states' => array(
      // Hide the settings when the itsyouonline_auto_create_account checkbox is disabled.
      'invisible' => array(
       ':input[name="itsyouonline_auto_create_account"]' => array('checked' => FALSE),
      ),
    ),
    '#required' => TRUE,
  );

  $form['registration']['itsyouonline_skip_link_wizard'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('itsyouonline_skip_link_wizard', 0),
    '#title' => t("Don't ask to link to an existing account."),
    '#description' => t('By enabling this option, for users who connect via itsyou.online and for who the itsyou.online account is not yet linked to a Drupal account, the user will not be asked whether he would like to link the account to an existing account or whether he would like to create a new account. Instead, the module will continue on the page that allows to create a new Drupal account. Depending on the previous selection, either a form will be shown via which the user can create his account, or a new a user is created automatically.'),
  );

  $form['authentication_mode'] = array(
    '#type' => 'fieldset',
    '#title' => t('Authentication mode'),
  );
  $form['authentication_mode']['itsyouonline_authentication_login_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Choose login mode'),
    '#options' => array(
      'itsyou' => t('itsyou.online only. The end-user can login <strong>only by using itsyou.online<strong>'),
      'mixed' => t('Mixed mode. The end-user can login with both itsyou.online and with username/password.'),
    ),
    '#default_value' => variable_get('itsyouonline_authentication_login_mode', 'mixed'),
    '#description' => t('In <i>itsyou.online only</i> mode, a user is forced to logon using itsyou.online. In <i>Mixed mode</i>, an end-user still can logon with username/password as well as by itsyou.online.'),
    '#required' => TRUE,
  );

  $form['authentication_mode']['itsyouonline_authentication_register_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Choose registration mode'),
    '#options' => array(
      'itsyou' => t('itsyou.online only. The end-user needs to be register <strong>only by using itsyou.online<strong>'),
      'mixed' => t('Mixed mode. The end-user can register with both itsyou.online as well as default drupal registration'),
    ),
    '#default_value' => variable_get('itsyouonline_authentication_register_mode', 'mixed'),
    '#description' => t('In <i>itsyou.online only</i> mode, a user is forced to register using itsyou.online. In <i>Mixed mode</i>, an end-user still can register with default drupal registration as well as by itsyou.online.'),
    '#required' => TRUE,
  );

  $form['language'] = array(
    '#type' => 'fieldset',
    '#title' => t('Language setting'),
  );

  $form['language']['itsyouonline_language_default'] = array(
    '#type' => 'checkbox',
    '#title' => t('Follow site language'),
    '#default_value' => variable_get('itsyouonline_language_default', true),
  );

  $form['language']['itsyouonline_language_force'] = array(
    '#type' => 'select',
    '#title' => t('Force language to'),
    '#options' => _itsyouonline_supported_languages(),
    '#states' => array(
      // Hide the settings when the itsyouonline_auto_create_account checkbox is disabled.
      'invisible' => array(
       ':input[name="itsyouonline_language_default"]' => array('checked' => TRUE),
      ),
    ),
    '#default_value' => variable_get('itsyouonline_language_force', null),
  );

  $form['features'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other features'),
  );

  $form['features']['itsyouonline_user_edit_redirect'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('itsyouonline_user_edit_redirect', 0),
    '#title' => t("On user edit, redirect user to itsyou.online."),
    '#description' => t('If user is linked to itsyou.online, on user edit redirect to itsyou.online profile.'),
  );

  $form['features']['itsyouonline_user_logout'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('itsyouonline_user_logout', 0),
    '#title' => t('On user logout, logout from itsyou.online'),
    '#description' => t('If user is logged in through itsyou.online the on user logout force user logout from itsyou.online'),
  );

  $form['features']['itsyouonline_user_hide_profile_info'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('itsyouonline_user_hide_profile_info', 0),
    '#title' => t('Don\'t show user details on profile page'),
    '#description' => t('If enable IYO user information will not be shown on user profile page'),
  );

  // @TODO: logout feature.

  return system_settings_form($form);
}

/**
 * Button style for itsyouonline.
 *
 * Page callback for 'admin/config/services/itsyouonline/button_style'.
 *
 * @return array
 *   Form generated by system_settings_form().
 */
function itsyouonline_admin_settings_button($form_state) {
  // @TODO: link option
  $forms = array(
    'login' => t('Login'),
    'register' => t('Register'),
    // 'link' => t('Link'),
  );

  $defaults = array(
    'login' => array(
      'text' => 'Login with itsyou.online',
    ),
    'register' => array(
      'text' => 'Signup with itsyou.online',
    ),
    /*
    'link' => array(
      'text' => 'Connect with itsyou.online',
    ),
    */
  );

  foreach ($forms as $key => $value) {
    $form[$key] = array(
      '#type' => 'fieldset',
      '#title' => t("@value button settings", array('@value' => $value)),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => t('Use the options below to modify link/button which is shown in the @key form.', array('@key' => $key)),
    );
    $form[$key]['itsyouonline_' . $key . '_button_text'] = array(
      '#type' => 'textfield',
      '#title' => t('Button title'),
      '#default_value' => variable_get('itsyouonline_' . $key . '_button_text', $defaults[$key]['text']),
      '#required' => TRUE,
      '#description' => t('Sets the button title.'),
    );

  }

  return system_settings_form($form);
}

/**
 * Theme callback for the itsyouonline_admin_settings_user_fields.
 *
 * The theme callback will format the $form data structure into a table and
 * add our tabledrag functionality. (Note that drupal_add_tabledrag should be
 * called from the theme layer, and not from a form declaration. This helps
 * keep template files clean and readable, and prevents tabledrag.js from
 * being added twice accidentally.
 *
 * @return string
 *   The rendered tabledrag form
 */
function theme_itsyouonline_admin_settings_user_fields($variables) {
  $form = $variables['form'];
  $header = array(t('Show field'), t('Show as'), t('Weight'), t('Map with'), t('Hide from user edit'));
  $rows = array();

  foreach (element_children($form['items']) as $key) {
    $element = &$form['items'][$key];
    $form['items'][$key]['weight']['#attributes']['class'] = array('weight-group');
    $row = array();
    $row[] = drupal_render($form['items'][$key]['selected']);
    $row[] = drupal_render($form['items'][$key]['title']);
    $row[] = drupal_render($form['items'][$key]['weight']) . drupal_render($form['items'][$key]['name']);

    if (isset($form['items'][$key]['field'])) {
      $row[] = drupal_render($form['items'][$key]['field']);
      $row[] = drupal_render($form['items'][$key]['field_edit_hide']);
    } else {
      $row[] = t('NA');
      $row[] = t('NA');
    }

    $rows[] = array('data' => $row, 'class' => array('draggable'));
  }
  $output = drupal_render($form['map_info']);

  $output .= theme('table', array(
    'header' => $header,
    'rows' => $rows,
    'attributes' => array('id' => 'draggable-table'),
  ));

  $output .= drupal_render_children($form);
  drupal_add_tabledrag('draggable-table', 'order', 'sibling', 'weight-group');
  return $output;
}

/**
 *
 * Shows the admin form which allows the administrator to specify which
 * attributes are being displayed on a user's profile page.
 */
function itsyouonline_admin_settings_user_fields($form_state) {
  $form = array();
  $profile_fields = array('' => t('-- select --'));
  $profile_field_exists = FALSE;
  $display_params = _itsyouonline_display_params();

  $user_fields = field_info_instances('user');

  if (!empty($user_fields['user'])) {
    foreach ($user_fields['user'] as $field_name => $field_details) {
      if ($field_details['widget']['type'] === 'text_textfield' ||
          $field_details['widget']['type'] === 'text_textarea') {
        $profile_fields[$field_name] = $field_details['label'];
        $profile_field_exists = TRUE;
      }
    }
  }

  $user_default_fields = array(
    'email' => array(
      'selected' => true,
      'title' => t('Email'),
      'weight' => 0,
    ),
    'firstname' => array(
      'selected' => true,
      'title' => t('Firstname'),
      'weight' => 0,
    ),
    'lastname' => array(
      'selected' => true,
      'title' => t('Lastname'),
      'weight' => 0,
    ),
  );

  $display_params_selected = variable_get('itsyouonline_user_display_fields', $user_default_fields);

  // Construct the form.
  $form = array();

  $form['map_info'] = array(
    '#type' => 'markup',
    '#markup' => t('User profile fields created of type \'text\' and \'textarea\' will be available for mapping')  
  );

  $form['items'] = array();
  $form['items']['#tree'] = TRUE;

  foreach ($display_params as $key => $title) {
    $form['items'][$key] = array(
      'selected' => array(
        '#type' => 'checkbox',
        '#title' => $title,
        '#default_value' => isset($display_params_selected[$key]) ? $display_params_selected[$key]['selected']: FALSE,
      ),
      'title' => array(
        '#type' => 'textfield',
        '#default_value' => isset($display_params_selected[$key]) ? $display_params_selected[$key]['title']: '',
      ),
      'weight' => array(
        '#type' => 'weight',
        '#delta' => count($display_params),
        '#default_value' => isset($display_params_selected[$key]) ? $display_params_selected[$key]['weight']: 0,
      ),
      'name' => array(
        '#type' => 'hidden',
        '#value' => $key,
      ),
    );

    if (!empty($profile_field_exists)) {
      $form['items'][$key]['field'] = array(
        '#type' => 'select',
        '#options' => $profile_fields,
        '#default_value' => (isset($display_params_selected[$key]) && isset($display_params_selected[$key]['field'])) ? $display_params_selected[$key]['field']: '',
      );

      $form['items'][$key]['field_edit_hide'] = array(
        '#type' => 'checkbox',
        '#default_value' => (isset($display_params_selected[$key]) && isset($display_params_selected[$key]['field_edit_hide'])) ? $display_params_selected[$key]['field_edit_hide']: FALSE
      );
    }
  }

  $form['sync_fields_each_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('Update synced field on each login'),
    '#default_value' => variable_get('itsyouonline_sync_fields_each_login', FALSE),
    '#description' => t("If not checked, mapped field will be synced from itsyou.online data only when user get logs in first time. Enable to sync for every time user logs in.")
  );

  $form['hide_account_email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide account email field from edit account'),
    '#default_value' => variable_get('itsyouonline_user_hide_email', FALSE),
  );

  $form['hide_account_password'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide update password from drupal account'),
    '#default_value' => variable_get('itsyouonline_user_hide_password', FALSE),
  );

  $data_scope_required = array('name', 'email');

  $form['data_scope'] = array(
    '#type' => 'fieldset',
    '#title' => t('User data scopes'),
  );

  $form['data_scope']['user_data_scopes'] = array(
    '#type' => 'checkboxes',
    '#options' => _itsyouonline_scope_params(),
    '#default_value' => variable_get('itsyouonline_user_scope_fields', $data_scope_required),
    '#title' => t('User attributes to retrieve from itsyou.online'),
    '#description' => t("Select the user attributes that you would like to retrieve from itsyou.online. Email and name are required. If you only want to authenticate users, then you don't have to select any other attributes."),
  );

  $form[] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function itsyouonline_admin_settings_user_fields_validate($form, $form_state) {
    if (empty($form_state['values']['user_data_scopes']['email']) ||
      empty($form_state['values']['user_data_scopes']['name'])) {
    form_set_error('user_data_scopes[email]', t('Email and name are required for scope field'));
  }
}

function itsyouonline_admin_settings_user_fields_submit($form, $form_state) {
  variable_set('itsyouonline_user_display_fields', $form_state['values']['items']);
  variable_set('itsyouonline_user_scope_fields', $form_state['values']['user_data_scopes']);

  variable_set('itsyouonline_sync_fields_each_login', $form_state['values']['sync_fields_each_login']);
  variable_set('itsyouonline_user_hide_email', $form_state['values']['hide_account_email']);
  variable_set('itsyouonline_user_hide_password', $form_state['values']['hide_account_password']);

  drupal_set_message(t('Settings has been saved successfully.'), 'status');
}
