<?php

define('ITSYOU_ONLINE_REQUEST_TIMEOUT', 60);

function itsyouonline_authorize_process($type) {
  $_SESSION['itsyouonline_token'] = user_password(32);
  $_SESSION['itsyouonline_action'] = $type;
  $_SESSION['itsyouonline_redirect'] =  isset($_GET['destination']) ? $_GET['destination'] : '';

  itsyouonline_authorize_callback();
}


function itsyouonline_authorize_callback() {
  $authorizeUrl = 'https://itsyou.online/v1/oauth/authorize?';
  $accesTokenUrl = 'https://itsyou.online/v1/oauth/access_token?';

  if (empty($_SESSION['itsyouonline_token']) ||
      empty($_SESSION['itsyouonline_action'])) {
    echo t('Invalid user session, please try again');
    return drupal_exit();
  }

  $clientId =  variable_get('itsyouonline_client_id', '');  
  $clientSecret = variable_get('itsyouonline_client_secret', '');
  $redirectUrl = url('itsyouonline/authorize', array('absolute' => TRUE));

  $state_array = array(
    'token' => $_SESSION['itsyouonline_token'],
    'action' => $_SESSION['itsyouonline_action']
  );

  $hash = _itsyouonline_authorize_encode_state($state_array);

  if (!isset($_GET['code'])) {

    $params = array(
      'client_id' => $clientId,
      'response_type' => 'code',
      'scope' => _itsyouonline_scope_params(true),
      'redirect_uri' => $redirectUrl,
      'state' => $hash
    );

    if ($_SESSION['itsyouonline_action'] === 'register') {
      $params['prefer'] = 'register';
    }

    $default_language = language_default('language');
    $all_languages = _itsyouonline_supported_languages();

    if (variable_get('itsyouonline_language_default', true) &&
        isset($all_languages[$default_language])) {
      $params['lang'] = $default_language;
    } else {
      $params['lang'] = variable_get('itsyouonline_language_force', 'en');
    }

    $paramQuery = http_build_query($params);

    header('Location: '. $authorizeUrl . $paramQuery);
  } else {
    if ($_GET['state'] !== $hash) {
      watchdog(
        'itsyouonline',
        t('Authorize callback error, oauth - state does not match'),
        array(),
        WATCHDOG_ERROR
      );

      echo t('Error while authentication, please try again');
      return drupal_exit();
    }

    $params = array(
      'client_id' => $clientId,
      'client_secret' => $clientSecret,
      'code' => $_GET['code'],
      'redirect_uri' => $redirectUrl,
      'state' => $hash
    );

    $paramQuery = http_build_query($params);

    // The HTTP request options.
    $options = array(
      'method' => 'POST',
      'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
      'timeout' => ITSYOU_ONLINE_REQUEST_TIMEOUT
      // 'data' => $paramQuery
    );


    $result = drupal_http_request($accesTokenUrl . $paramQuery, $options);

    switch ($result->code) {
      case 200:
      case 301:
      case 302:
        $resp = json_decode($result->data);

        if (json_last_error()) {
          watchdog(
            'itsyouonline',
            t('Authorize callback error, oauth - error while loading getting access key'),
            array(),
            WATCHDOG_ERROR
          );

          return t('Error while authenticating user with itsyou.');


        }
        
        return _itsyouonline_authorize_post_callback($resp, $state_array['action']);

      break;

      default:
        watchdog(
          'itsyouonline',
          t('Authorize callback error, oauth - error while loading getting access key'),
          array(),
          WATCHDOG_ERROR
        );

        return t('Error while authenticating user with itsyou.');
    }
  }
}

function itsyouonline_get_user_info($username, $auth_data) {
  $getInfoRestUrl = 'https://itsyou.online/api/users/{{USERNAME}}/info?access_token={{TOKEN}}';
  $getInfoRestUrl = str_replace("{{USERNAME}}", $username, $getInfoRestUrl);
  $getInfoRestUrl = str_replace("{{TOKEN}}", $auth_data->access_token, $getInfoRestUrl);

  $userinfo = NULL;

  $options = array(
    'method' => 'GET',
    'timeout' => ITSYOU_ONLINE_REQUEST_TIMEOUT
    // 'headers' => array('Authorization' => 'Bearer ' . $resp->access_token),
  );

  $result = drupal_http_request($getInfoRestUrl, $options);

  switch ($result->code) {
    case 200:
    case 301:
    case 302:
      $resp = json_decode($result->data, TRUE);

      if (json_last_error()) {
        watchdog(
          'itsyouonline',
          t('Get user info error - error while loading itsyou data for user @user'),
          array('@user' => $username),
          WATCHDOG_ERROR
        );

      } else {
        $userinfo = $resp;
      }

      break;

    default:
      watchdog(
        'itsyouonline',
        t('Get user info error - error while loading itsyou data for user @user'),
        array('@user' => $username),
        WATCHDOG_ERROR
      );
  }

  return $userinfo;
}

function _itsyouonline_authorize_post_callback($resp, $process_type) {
  $return = array();
  $return['userinfo'] = itsyouonline_get_user_info($resp->info->username, $resp);

  if (!$return['userinfo']) {
    return t('Error while loading itsyou user details');
  }

  $return['authdata'] = $resp;

  itsyouonline_process_user($return, $process_type);
}

/**
 * Encodes an array into a Base64 string.
 *
 * This function encodes an array into a Base64 string so that it can be used
 * as a value to the state parameter
 *
 * @param array $state_array
 *   The array containing the values which have to be encoded.
 *
 * @return string
 *   The Base64 representation of the array.
 */
function _itsyouonline_authorize_encode_state(array $state_array) {
  return base64_encode(json_encode($state_array));
}

/**
 * Decodes a Base64 string into an array.
 *
 * This function decodes a Base64 string into an array. 
 *
 * @param string $state_string
 *   The Base64 string received by the callback function.
 *
 * @return array
 *   The decoded Base64 string.
 */
function _itsyouonline_decode_state_array($state_string) {
  return json_decode(base64_decode($state_string, TRUE), TRUE);
}
