<?php

function itsyouonline_integration_process($type, $info, $auth) {
  $itsyouonline_uid = $info['username'];
  $itsyouonline_redirect = isset($_SESSION['itsyouonline_redirect']) ? $_SESSION['itsyouonline_redirect'] : '';

  // Update the user data in the database.
  db_delete('itsyouonline_user_data')
    ->condition('itsyou_uid', $itsyouonline_uid)
    ->execute();

  // Insert the user data using a multi-insert query.
  $query = db_insert('itsyouonline_user_data')
    ->fields(array('itsyou_uid', 'attribute_key', 'attribute_value'));

  foreach ($info as $key => $value) {
    $query->values(array(
      'itsyou_uid' => $itsyouonline_uid,
      'attribute_key' => $key,
      'attribute_value' => $value,
    ));
  }

  $query->execute();

  // Check whether this end-user is already linked to a Drupal user.
  $query = db_select('itsyouonline_user_link', 'iul')
    ->condition('iul.itsyou_uid', $itsyouonline_uid)
    ->fields('iul', array('drupal_uid', 'is_active'));
  $result = $query->execute()->fetch();

  if ($result) {
    // The itsyou.online end-user is already linked to an existing Drupal
    // user, let's authenticate the Drupal user.
    $account = user_load($result->drupal_uid);

    // Store new auth data
    db_update('itsyouonline_user_link')
    ->fields(array(
      'auth_data' => serialize($auth),
      'is_active' => 1,
      'updated' => REQUEST_TIME
    ))
    ->condition('itsyou_uid', $itsyouonline_uid)
    ->execute();

    // The following authentication flow is copied from the core openid module.
    if (!variable_get('user_email_verification', TRUE) || $account->login) {
      // Check if user is blocked.
      $state['values']['name'] = $account->name;
      user_login_name_validate(array(), $state);
      if (!form_get_errors()) {
        // Load global $user and perform final login tasks.
        _itsyouonline_update_user_fields($result, $account);

        $form_state['uid'] = $account->uid;
        user_login_submit(array(), $form_state);
      }
    }
    else {
      drupal_set_message(t('You must validate your email address for this account before logging in via itsyou.online.'), 'warning');
    }

    // Redirect the user to the front page/destination page
    _itsyouonline_integration_redirect($itsyouonline_redirect);

    return;
  }
  else {

    // The end-user is not yet linked to an existing Drupal
    // user.
    if (user_is_logged_in() && $type == 'link') {
      //@TODO: Cleanup the session data: the link code is already consumed.

      global $user;

      // Link to currently logged on user.
      $fields = array(
        'drupal_uid' => $user->uid,
        'itsyou_uid' => $itsyouonline_uid,
        'auth_data' => serialize($auth),
        'is_active' => 1,
        'updated' => REQUEST_TIME
      );
      try {
        db_insert('itsyouonline_user_link')->fields($fields)->execute();
      }
      catch (Exception $e) {
        watchdog_exception('itsyouonline', $e);
        drupal_set_message(t('An error occurred while linking the user to itsyou.online'), 'error');
        _itsyouonline_integration_redirect();
        return;
      }

      drupal_set_message(t('The user has been successfully linked to itsyou.online'));

      //@see: should connect back to org?

      // Redirect to the user edit form

      if ($_SESSION['itsyouonline_redirect']) {
        _itsyouonline_integration_redirect($_SESSION['itsyouonline_redirect']);
      } else {
        _itsyouonline_integration_redirect('user/' . $user->uid);
      }

      return;
    }
    else {

      if ($type && ($type == 'register')) {
        _itsyouonline_integration_redirect('itsyouonline/link/new_user');
      }
      else {
        if (variable_get('itsyouonline_skip_link_wizard', 0) == 0) {
          _itsyouonline_integration_redirect('itsyouonline/link');
        } else {
          _itsyouonline_integration_redirect('itsyouonline/link/new_user');
        }
      }
      return;
    }
  }
}


function _itsyouonline_integration_redirect($url = NULL) {
  drupal_goto($url);

  /*
  $url = $url ? url($url) : url();
    echo '<script type="text/javascript">
        window.opener.oauth2Callback("' . $url . '");
        window.close();
    </script>';
    drupal_exit();
  */
}